@@ -8,19 +8,19 @@
 # pyre-ignore-all-errors[2]: Allow `Any` in type annotations
 
 import base64
+import json
 import logging
 import struct
-from collections import defaultdict
 from dataclasses import asdict, dataclass
 from enum import Enum
-from typing import Any, cast, Dict, List, Optional, Tuple, TypeVar, Union
+from typing import Any, ClassVar, Dict, List, Optional, Tuple, TypeVar, Union
 
 import yaml
 
 try:
-    from yaml import CSafeDumper as Dumper, CSafeLoader as Loader
+    from yaml import CSafeLoader as Loader
 except ImportError:
-    from yaml import SafeDumper as Dumper, SafeLoader as Loader
+    from yaml import SafeLoader as Loader
 
 logger: logging.Logger = logging.getLogger(__name__)
 
@@ -37,6 +37,12 @@ class Entry:
 
     type: str
 
+    @classmethod
+    def from_yaml_obj(cls, yaml_obj: Any) -> "Entry":
+        if "type" in yaml_obj:
+            del yaml_obj["type"]
+        return cls(**yaml_obj)
+
 
 @dataclass
 class TensorEntry(Entry):
@@ -79,6 +85,11 @@ class Shard:
     sizes: List[int]
     tensor: TensorEntry
 
+    @classmethod
+    def from_yaml_obj(cls, yaml_obj: Any) -> "Shard":
+        yaml_obj["tensor"] = TensorEntry.from_yaml_obj(yaml_obj["tensor"])
+        return cls(**yaml_obj)
+
 
 @dataclass
 class ShardedTensorEntry(Entry):
@@ -89,23 +100,13 @@ class ShardedTensorEntry(Entry):
         self.shards = shards
 
     @classmethod
-    def from_yaml(cls, entry: Any) -> "ShardedTensorEntry":
-        shards = [
-            Shard(
-                offsets=shard["offsets"],
-                sizes=shard["sizes"],
-                tensor=TensorEntry(
-                    location=shard["tensor"]["location"],
-                    serializer=shard["tensor"]["serializer"],
-                    dtype=shard["tensor"]["dtype"],
-                    shape=shard["tensor"]["shape"],
-                    replicated=shard["tensor"]["replicated"],
-                    byte_range=shard["tensor"].get("byte_range"),
-                ),
-            )
-            for shard in entry["shards"]
+    def from_yaml_obj(cls, yaml_obj: Any) -> "ShardedTensorEntry":
+        if "type" in yaml_obj:
+            del yaml_obj["type"]
+        yaml_obj["shards"] = [
+            Shard.from_yaml_obj(shard) for shard in yaml_obj["shards"]
         ]
-        return cls(shards=shards)
+        return cls(**yaml_obj)
 
 
 @dataclass
@@ -125,31 +126,13 @@ class ChunkedTensorEntry(Entry):
         self.replicated = replicated
 
     @classmethod
-    def from_yaml(cls, entry: Any) -> "ChunkedTensorEntry":
-        dtype = entry["dtype"]
-        replicated = entry["replicated"]
-        chunks = [
-            Shard(
-                offsets=chunk["offsets"],
-                sizes=chunk["sizes"],
-                tensor=TensorEntry(
-                    location=chunk["tensor"]["location"],
-                    serializer=chunk["tensor"]["serializer"],
-                    dtype=chunk["tensor"]["dtype"],
-                    shape=chunk["tensor"]["shape"],
-                    replicated=chunk["tensor"]["replicated"],
-                    byte_range=chunk["tensor"].get("byte_range"),
-                ),
-            )
-            for chunk in entry["chunks"]
+    def from_yaml_obj(cls, yaml_obj: Any) -> "ChunkedTensorEntry":
+        if "type" in yaml_obj:
+            del yaml_obj["type"]
+        yaml_obj["chunks"] = [
+            Shard.from_yaml_obj(shard) for shard in yaml_obj["chunks"]
         ]
-        shape = entry["shape"]
-        return cls(
-            dtype=dtype,
-            shape=shape,
-            chunks=chunks,
-            replicated=replicated,
-        )
+        return cls(**yaml_obj)
 
 
 @dataclass
@@ -186,9 +169,9 @@ class DictEntry(Entry):
 
 @dataclass
 class OrderedDictEntry(Entry):
-    keys: List[str]
+    keys: List[Union[str, int]]
 
-    def __init__(self, keys: List[str]) -> None:
+    def __init__(self, keys: List[Union[str, int]]) -> None:
         super().__init__(type="OrderedDict")
         self.keys = keys
 
@@ -211,6 +194,8 @@ class PrimitiveEntry(Entry):
     readable: for ease of inspection for certain types
     """
 
+    supported_types: ClassVar[List[str]] = [t.value for t in PrimitiveType]
+
     serialized_value: str
     replicated: bool
     readable: Optional[str]
@@ -247,10 +232,6 @@ class PrimitiveEntry(Entry):
             f"Unable to get deserialized value for {self.serialized_value}"
         )
 
-    @classmethod
-    def supported_types(cls) -> List[str]:
-        return [t.value for t in PrimitiveType]
-
     @classmethod
     def _serialize(cls, type_name: str, obj: Any) -> str:
         if type_name == "int":
@@ -270,7 +251,7 @@ class PrimitiveEntry(Entry):
     @classmethod
     def from_object(cls, obj: Any) -> "PrimitiveEntry":
         type_name = type(obj).__name__
-        if type_name not in cls.supported_types():
+        if type_name not in cls.supported_types:
             raise TypeError(f"Unsupported primitive obj of type {type_name}")
 
         serialized_value = cls._serialize(type_name, obj)
@@ -278,17 +259,15 @@ class PrimitiveEntry(Entry):
         return PrimitiveEntry(type_name, serialized_value, False, readable_value)
 
     @classmethod
-    def from_serialized(
+    def from_yaml_obj(
         cls,
-        type_name: str,
-        serialized_value: str,
-        replicated: bool,
-        readable: Optional[str],
+        yaml_obj: Any,
     ) -> "PrimitiveEntry":
-        if type_name not in cls.supported_types():
+        type_name = yaml_obj["type"]
+        if type_name not in cls.supported_types:
             raise TypeError(f"Unsupported primitive obj of type {type_name}")
-
-        return PrimitiveEntry(type_name, serialized_value, replicated)
+        del yaml_obj["readable"]
+        return cls(**yaml_obj)
 
 
 T = TypeVar("T", bound=Entry)
@@ -302,94 +281,41 @@ class SnapshotMetadata:
     manifest: Manifest
 
     def to_yaml(self) -> str:
-        return yaml.dump(asdict(self), sort_keys=False, Dumper=Dumper)
+        # When the number of entries in the snapshot metadata is large, yaml
+        # serialization becomes slow and there's little room for optimization.
+        # Since the snapshot metadata can be dumped as json and json is a
+        # subset of yaml, using json.dumps() here to help with the
+        # serialization performance without needing to deprecate yaml.
+        return json.dumps(asdict(self), sort_keys=False, indent=2)
 
     @classmethod
     def from_yaml(cls, yaml_str: str) -> "SnapshotMetadata":
         d = yaml.load(yaml_str, Loader=Loader)
         manifest: Manifest = {}
-        for path, entry in d["manifest"].items():
-            type_name = entry["type"]
-            del entry["type"]
+        for path, yaml_obj in d["manifest"].items():
+            type_name = yaml_obj["type"]
             if type_name == "list":
-                manifest[path] = ListEntry(**entry)
+                manifest[path] = ListEntry.from_yaml_obj(yaml_obj)
             elif type_name == "dict":
-                manifest[path] = DictEntry(**entry)
+                manifest[path] = DictEntry.from_yaml_obj(yaml_obj)
             elif type_name == "OrderedDict":
-                manifest[path] = OrderedDictEntry(**entry)
-            elif type_name in PrimitiveEntry.supported_types():
-                manifest[path] = PrimitiveEntry.from_serialized(type_name, **entry)
+                manifest[path] = OrderedDictEntry.from_yaml_obj(yaml_obj)
+            elif type_name in PrimitiveEntry.supported_types:
+                manifest[path] = PrimitiveEntry.from_yaml_obj(yaml_obj)
             elif type_name == "Tensor":
-                manifest[path] = TensorEntry(**entry)
+                manifest[path] = TensorEntry.from_yaml_obj(yaml_obj)
             elif type_name == "ShardedTensor":
-                manifest[path] = ShardedTensorEntry.from_yaml(entry=entry)
+                manifest[path] = ShardedTensorEntry.from_yaml_obj(yaml_obj)
             elif type_name == "ChunkedTensor":
-                manifest[path] = ChunkedTensorEntry.from_yaml(entry=entry)
+                manifest[path] = ChunkedTensorEntry.from_yaml_obj(yaml_obj)
             elif type_name == "object":
-                manifest[path] = ObjectEntry(**entry)
+                manifest[path] = ObjectEntry.from_yaml_obj(yaml_obj)
         d["manifest"] = manifest
         return cls(**d)
 
 
-def get_available_entries(manifest: Manifest, rank: int) -> Manifest:
-    """
-    Prepare available entries to load from for the rank.
-
-    Given a global manifest, prepare available entries to load from for the
-    rank according to the following rules:
-
-        per-rank: The entry is only made available to the rank saved it.
-        replicated: The entry is made available to all ranks.
-        sharded: Entries are first merged across all ranks then made available
-            to all ranks.
-
-    The function will not return any container entries which are only used to
-    reconstruct the orginal state dict.
-
-    Args:
-        manifest: The global manifest.
-        rank: The rank of the current process.
-
-    Returns:
-        The local manifest for the rank.
-    """
-    logical_path_to_rank_to_entry: Dict[str, Dict[int, Entry]] = defaultdict(dict)
-    for path, entry in manifest.items():
-        tokens = path.split("/")[0]
-        entry_rank = int(tokens[0])
-        logical_path = "/".join(path.split("/")[1:])
-        logical_path_to_rank_to_entry[logical_path][entry_rank] = entry
-
-    local_manifest = {}
-    for logical_path, rank_to_entry in logical_path_to_rank_to_entry.items():
-        entries = list(rank_to_entry.values())
-
-        # The logical path corresponds to a replicated entry
-        if is_replicated(entries[0]):
-            local_manifest.setdefault(logical_path, entries[0])
-        # The logical path corresponds to a ShardedTensorEntry
-        elif isinstance(entries[0], ShardedTensorEntry):
-            # TODO: on save, we should enforce the following invariants that if
-            # a logical path on one rank is a ShardedTensorEntry, the logical
-            # path on all ranks that has the logical path is a ShardedTensorEntry.
-            local_manifest[logical_path] = ShardedTensorEntry(
-                shards=[
-                    shard
-                    for entry in entries
-                    for shard in cast(ShardedTensorEntry, entry).shards
-                ]
-            )
-        # The logical path corresponds to a per-rank entry
-        elif rank in rank_to_entry:
-            # Skip container entries
-            if is_container_entry(rank_to_entry[rank]):
-                continue
-            local_manifest[logical_path] = rank_to_entry[rank]
-        # The logical path doesn't exist for this rank
-        else:
-            pass
-
-    return local_manifest
+def is_dict_entry(entry: Entry) -> bool:
+    return isinstance(entry, (DictEntry, OrderedDictEntry))
 
 
 def is_replicated(entry: Entry) -> bool:
